package com.hundsun.hybrid;

import java.util.ArrayList;
import java.util.List;

import android.app.Activity;
import android.content.Intent;
import android.content.res.Configuration;
import android.os.Bundle;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.MenuItem;

public class BaseActivity extends Activity {
	private ActivityListeners listeners = new ActivityListeners();

	public void addListener(ActivityListener listener) {
		this.listeners.addListener(listener);
	}

	@Override
	public void onConfigurationChanged(Configuration newConfig) {
		super.onConfigurationChanged(newConfig);
		this.listeners.fireOnConfigurationChanged(newConfig);
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		return this.listeners.fireOnCreateOptionsMenu(menu);
	}

	@Override
	protected void onDestroy() {
		this.listeners.fireOnDestroy();
		super.onDestroy();
	}

	@Override
	public boolean onKeyDown(int keyCode, KeyEvent event) {
		return this.listeners.fireOnActivityKeyDown(keyCode, event);
	}

	@Override
	protected void onNewIntent(Intent intent) {
		super.onNewIntent(intent);
		this.listeners.fireOnNewIntent(intent);
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		return this.listeners.fireOnOptionsItemSelected(item);
	}

	@Override
	protected void onPause() {
		super.onPause();
		this.listeners.fireOnPause();
	}

	@Override
	public boolean onPrepareOptionsMenu(Menu menu) {
		return this.listeners.fireOnPrepareOptionsMenu(menu);
	}

	@Override
	protected void onResume() {
		super.onResume();
		this.listeners.fireOnResume();
	}

	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		super.onActivityResult(requestCode, resultCode, data);
		this.listeners.fireOnActivityResult(requestCode, resultCode, data);
	}
	
	

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		this.listeners.fireOnActivityCreate(savedInstanceState);
	}



	private class ActivityListeners {
		private List<ActivityListener> listeners = new ArrayList<ActivityListener>(1);

		public void addListener(ActivityListener listener) {
			if (listener != null) {
				this.listeners.add(listener);
			}
		}
		
		public void fireOnActivityCreate(Bundle savedInstanceState) {
			for (ActivityListener listener : this.listeners) {
				listener.onActivityCreate(savedInstanceState);
			}
		}

		public void fireOnDestroy() {
			for (ActivityListener listener : this.listeners) {
				listener.onActivityDestroy();
			}
		}

		public void fireOnPause() {
			for (ActivityListener listener : this.listeners) {
				listener.onActivityPause();
			}
		}

		public void fireOnResume() {
			for (ActivityListener listener : this.listeners) {
				listener.onActivityResume();
			}
		}

		public boolean fireOnCreateOptionsMenu(Menu menu) {
			for (ActivityListener listener : this.listeners) {
				ActionResult<Boolean> result = listener.onCreateOptionsMenu(menu);
				if (result != null && !result.needContinue()) {
					return result.getReturnValue();
				}
			}
			return BaseActivity.super.onCreateOptionsMenu(menu);
		}

		public boolean fireOnActivityKeyDown(int keyCode, KeyEvent event) {
			for (ActivityListener listener : this.listeners) {
				ActionResult<Boolean> result = listener.onActivityKeyDown(keyCode, event);
				if (result != null && !result.needContinue()) {
					return result.getReturnValue();
				}
			}
			return BaseActivity.super.onKeyDown(keyCode, event);
		}

		public boolean fireOnOptionsItemSelected(MenuItem item) {
			for (ActivityListener listener : this.listeners) {
				ActionResult<Boolean> result = listener.onOptionsItemSelected(item);
				if (result != null && !result.needContinue()) {
					return result.getReturnValue();
				}
			}
			return BaseActivity.super.onOptionsItemSelected(item);
		}

		public boolean fireOnPrepareOptionsMenu(Menu menu) {
			for (ActivityListener listener : this.listeners) {
				ActionResult<Boolean> result = listener.onPrepareOptionsMenu(menu);
				if (result != null && !result.needContinue()) {
					return result.getReturnValue();
				}
			}
			return BaseActivity.super.onPrepareOptionsMenu(menu);
		}

		public void fireOnConfigurationChanged(Configuration newConfig) {
			for (ActivityListener listener : this.listeners) {
				listener.onActivityConfigurationChanged(newConfig);
			}
		}

		public void fireOnNewIntent(Intent intent) {
			for (ActivityListener listener : this.listeners) {
				listener.onNewIntent(intent);
			}
		}

		public void fireOnActivityResult(int requestCode, int resultCode, Intent intent) {
			for (ActivityListener listener : this.listeners) {
				listener.onActivityResult(requestCode, resultCode, intent);
			}
		}
	}
}
