package com.hundsun.hybrid;

import java.lang.reflect.Method;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import android.os.Bundle;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.widget.Button;
import android.widget.LinearLayout;
import android.widget.TextView;

import com.hundsun.hybrid.api.Plugin;
import com.hundsun.hybrid.api.PluginResult;
import com.hundsun.hybrid.utils.JsonUtils;
import com.hundsun.hybrid.utils.ResUtil;
import com.hundsun.hybrid.widget.HybridLayout;
import com.hundsun.hybrid.widget.HybridNaviBar;

public class HybridActivity extends BaseActivity {
	private HybridView hybridView;
	private LinearLayout header;
	private LinearLayout footer;

	private Button naviButton1;
	private Button naviButton2;
	private TextView naviTitle;
	private LinearLayout naviBar;

	private ViewGroup windowTitle;
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		this.setTheme(ResUtil.getResourceId(this, "R.style.hl_theme_hybrid"));
		this.requestWindowFeature(Window.FEATURE_CUSTOM_TITLE);
		HybridLayout layout = new HybridLayout(this);
		this.setContentView(layout);

		int naviBarResId = ResUtil.getResourceId(this, "R.layout.hl_navi_bar");

		if (naviBarResId != 0) {
			this.getWindow().setFeatureInt(Window.FEATURE_CUSTOM_TITLE, ResUtil.getResourceId(this, "R.layout.hl_navi_bar"));

			this.naviBar = (LinearLayout) this.findViewById(ResUtil.getResourceId(this, "R.id.hl_navi_bar"));
			this.naviButton1 = (Button) this.findViewById(ResUtil.getResourceId(this, "R.id.hl_navi_button1"));
			if (this.naviButton1.getText() == null || this.naviButton1.getText().toString().trim().length() == 0) {
				this.naviButton1.setVisibility(View.GONE);
			}
			this.naviButton2 = (Button) this.findViewById(ResUtil.getResourceId(this, "R.id.hl_navi_button2"));
			if (this.naviButton2.getText() == null || this.naviButton2.getText().toString().trim().length() == 0) {
				this.naviButton2.setVisibility(View.GONE);
			}
			this.naviTitle = (TextView) this.findViewById(ResUtil.getResourceId(this, "R.id.hl_text_title"));
		} else {
			HybridNaviBar naviBar = new HybridNaviBar(this);
			this.naviButton1 = naviBar.getNaviButton1();
			this.naviButton2 = naviBar.getNaviButton2();
			this.naviTitle = naviBar.getNaviTitle();
			this.naviBar = naviBar;
			try {
				int titleContainerId = (Integer) Class.forName("com.android.internal.R$id").getField("title_container").get(null);
				this.windowTitle = (ViewGroup) (this.getWindow().findViewById(titleContainerId));
				this.windowTitle.addView(naviBar);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}

		this.hybridView = layout.getHybridView();
		this.header = layout.getHeader();
		this.footer = layout.getFooter();

	}

	public void showNavBar(JSONArray args, String callbackId, Plugin plugin) {
		if (this.windowTitle != null) {
			this.runOnUiThread(new Runnable() {
				@Override
				public void run() {
					HybridActivity.this.windowTitle.setVisibility(View.VISIBLE);
				}
			});
		}

	}

	public void hideNavBar(JSONArray args, String callbackId, Plugin plugin) {
		if (this.windowTitle != null) {
			this.runOnUiThread(new Runnable() {
				@Override
				public void run() {
					HybridActivity.this.windowTitle.setVisibility(View.GONE);
				}
			});
		}
	}

	public PluginResult setNavBarTitle(JSONArray args, String callbackId, Plugin plugin) {
		if (args == null || args.length() == 0) {
			return PluginResult.JSON_EXCEPTION;
		}
		if (this.naviTitle != null) {
			try {
				final String title = args.getString(0);
				this.runOnUiThread(new Runnable() {
					@Override
					public void run() {
						HybridActivity.this.naviTitle.setText(title);
						HybridActivity.this.naviTitle.setVisibility(View.VISIBLE);
					}
				});
			} catch (JSONException e) {
				e.printStackTrace();
				return PluginResult.JSON_EXCEPTION;
			}
		}
		return PluginResult.OK;
	}

	public void hideLeftNavButton(JSONArray args, String callbackId, Plugin plugin) {
		if (this.naviButton1 != null) {
			this.runOnUiThread(new Runnable() {
				@Override
				public void run() {
					HybridActivity.this.naviButton1.setVisibility(View.GONE);
				}
			});
		}
	}

	public PluginResult setupLeftNavButton(JSONArray args, final String callbackId, final Plugin plugin) {
		if (args.length() == 0) {
			return PluginResult.JSON_EXCEPTION;
		}

		try {
			final JSONObject json = args.getJSONObject(0);
			this.runOnUiThread(new Runnable() {
				@Override
				public void run() {
					HybridActivity.this.naviButton1.setText(JsonUtils.getStr(json, "text", "back"));
					HybridActivity.this.naviButton1.setVisibility(View.VISIBLE);
				}
			});
			this.naviButton1.setOnClickListener(new Button.OnClickListener() {
				@Override
				public void onClick(View v) {
					JSONObject r = new JSONObject();
					try {
						r.put("onClicked", true);
					} catch (JSONException e) {
						e.printStackTrace();
					}
					PluginResult result = new PluginResult(PluginResult.Status.OK, r);
					result.setKeepCallback(true);
					plugin.success(result, callbackId);
				}
			});
		} catch (JSONException e) {
			e.printStackTrace();
			return PluginResult.JSON_EXCEPTION;
		}
		PluginResult result = new PluginResult(PluginResult.Status.OK);
		result.setKeepCallback(true);
		return result;
	}

	public PluginResult setupRightNavButton(JSONArray args, final String callbackId, final Plugin plugin) {
		if (args.length() == 0) {
			return PluginResult.JSON_EXCEPTION;
		}
		try {
			final JSONObject json = args.getJSONObject(0);

			this.naviButton2.setOnClickListener(new Button.OnClickListener() {
				@Override
				public void onClick(View v) {
					JSONObject r = new JSONObject();
					try {
						r.put("onClicked", true);
					} catch (JSONException e) {
						e.printStackTrace();
					}
					PluginResult result = new PluginResult(PluginResult.Status.OK, r);
					result.setKeepCallback(true);
					plugin.success(result, callbackId);
				}
			});

			this.runOnUiThread(new Runnable() {
				@Override
				public void run() {
					HybridActivity.this.naviButton2.setText(JsonUtils.getStr(json, "text", "back"));
					HybridActivity.this.naviButton2.setVisibility(View.VISIBLE);
				}
			});
		} catch (JSONException e) {
			e.printStackTrace();
			return PluginResult.JSON_EXCEPTION;
		}
		PluginResult result = new PluginResult(PluginResult.Status.OK);
		result.setKeepCallback(true);
		return result;
	}

	public void hideRightNavButton(JSONArray args, String callbackId, Plugin plugin) {
		if (this.naviButton2 != null) {
			this.runOnUiThread(new Runnable() {
				@Override
				public void run() {
					HybridActivity.this.naviButton2.setVisibility(View.GONE);
				}
			});
		}
	}

	public PluginResult turnPageAnimation(JSONArray args, String callbackId, Plugin plugin) {
		if (args == null || args.length() == 0) {
			return PluginResult.JSON_EXCEPTION;
		}
		PluginResult result = new PluginResult(PluginResult.Status.OK);
		return result;
	}

	public void showToolBarMenu(JSONArray args, String callbackId, Plugin plugin) {
		if (this.footer != null) {
			this.runOnUiThread(new Runnable() {
				@Override
				public void run() {
					HybridActivity.this.footer.setVisibility(View.VISIBLE);
				}
			});
		}
	}

	public void hideToolBarMenu(JSONArray args, String callbackId, Plugin plugin) {
		if (this.footer != null) {
			this.runOnUiThread(new Runnable() {
				@Override
				public void run() {
					HybridActivity.this.footer.setVisibility(View.GONE);
				}
			});
		}
	}

	public PluginResult exec(String action, JSONArray args, String callbackId, Plugin plugin) {
		Object ret = null;
		Method doMethod;
		try {
			// this.getClass().getDeclaredMethod(name, parameterTypes)
			doMethod = this.getClass().getMethod(action, JSONArray.class, String.class, Plugin.class);
			if (doMethod != null) {
				ret = doMethod.invoke(this, args, callbackId, plugin);
			}
			Class<?> cls = doMethod.getReturnType();
			if (ret != null && cls.equals(PluginResult.class)) {
				return (PluginResult) ret;
			} else {
				return PluginResult.OK;
			}
		} catch (Exception e) {
			e.printStackTrace();
		}

		return PluginResult.OK;
	}

	public HybridView getHybridView() {
		return hybridView;
	}

	public LinearLayout getHeader() {
		return header;
	}

	public LinearLayout getFooter() {
		return footer;
	}

	public Button getNaviButton1() {
		return naviButton1;
	}

	public Button getNaviButton2() {
		return naviButton2;
	}

	public TextView getNaviTitle() {
		return naviTitle;
	}

	public LinearLayout getNaviBar() {
		return naviBar;
	}
}
