package com.hundsun.hybrid;

import java.util.HashMap;

import org.json.JSONException;
import org.json.JSONObject;

import com.hundsun.hybrid.api.Hybrid;
import com.hundsun.hybrid.api.IPlugin;
import com.hundsun.hybrid.api.LOG;
import com.hundsun.hybrid.utils.IntentUtil;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.res.Configuration;
import android.graphics.Color;
import android.media.AudioManager;
import android.os.Bundle;
import android.os.Handler;
import android.util.AttributeSet;
import android.view.Display;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.view.WindowManager;
import android.webkit.WebViewClient;
import android.widget.LinearLayout;

public class HybridView extends LinearLayout implements Hybrid, ActivityListener {
	public static final String TAG = HybridView.class.getSimpleName();

	// The webview for our app
	protected HybridWebView webView;
	protected HybridWebViewClient webViewClient;

	protected boolean cancelLoadUrl = false;
	protected ProgressDialog spinnerDialog = null;

	// The initial URL for our app
	// ie http://server/path/index.html#abc?query
	// private String url = null;

	private static int ACTIVITY_STARTING = 0;
	private static int ACTIVITY_RUNNING = 1;
	private static int ACTIVITY_EXITING = 2;
	private int activityState = 0; // 0=starting, 1=running (after 1st resume),
									// 2=shutting down

	// The base of the initial URL for our app.
	// Does not include file name. Ends with /
	// ie http://server/path/
	String baseUrl = null;

	// Plugin to call when activity result is received
	protected IPlugin activityResultCallback = null;
	protected boolean activityResultKeepRunning;

	// Default background color for activity
	// (this is not the color for the webview, which is set in HTML)
	private int backgroundColor = Color.WHITE;

	/*
	 * The variables below are used to cache some of the activity properties.
	 */

	// Draw a splash screen using an image located in the drawable resource
	// directory.
	// This is not the same as calling super.loadSplashscreen(url)
	protected int splashscreen = 0;
	protected int splashscreenTime = 0;

	// LoadUrl timeout value in msec (default of 20 sec)
	protected int loadUrlTimeoutValue = 20000;

	// Keep app running when pause is received. (default = true)
	// If true, then the JavaScript and native code continue to run in the
	// background
	// when another application (activity) is started.
	protected boolean keepRunning = true;

	private int oldHeight = 0; // Need to save the old height as not to send
								// redundant events
	private int oldWidth = 0; // Need to save old width for orientation change
	private int screenWidth = 0;
	private int screenHeight = 0;

	private BaseActivity activity;

	public HybridView(Context context) {
		super(context);
		if (BaseActivity.class.isInstance(context)) {
			this.activity = BaseActivity.class.cast(context);
			this.activity.addListener(this);
			this.onActivityCreate(null);
		} else {
			throw new RuntimeException("class '" + context.getClass().getName() + "' must extend from BaseActivity to work");
		}
	}

	public HybridView(Context context, AttributeSet attrs) {
		super(context, attrs);
		if (BaseActivity.class.isInstance(context)) {
			this.activity = BaseActivity.class.cast(context);
			this.activity.addListener(this);
			this.onActivityCreate(null);
		} else {
			throw new RuntimeException("class '" + context.getClass().getName() + "' must extend from BaseActivity to work");
		}
	}

//	public HybridView(Context context, AttributeSet attrs, int defStyle) {
//		super(context, attrs, defStyle);
//		if (BaseActivity.class.isInstance(context)) {
//			this.activity = BaseActivity.class.cast(context);
//			this.activity.addListener(this);
//			this.onActivityCreate(null);
//		} else {
//			throw new RuntimeException("class '" + context.getClass().getName() + "' must extend from BaseActivity to work");
//		}
//	}

	/**
	 * Sets the authentication token.
	 * 
	 * @param authenticationToken
	 * @param host
	 * @param realm
	 */
	public void setAuthenticationToken(AuthenticationToken authenticationToken, String host, String realm) {
		if (this.webView != null && this.webView.viewClient != null) {
			this.webView.viewClient.setAuthenticationToken(authenticationToken, host, realm);
		}
	}

	/**
	 * Removes the authentication token.
	 * 
	 * @param host
	 * @param realm
	 * 
	 * @return the authentication token or null if did not exist
	 */
	public AuthenticationToken removeAuthenticationToken(String host, String realm) {
		if (this.webView != null && this.webView.viewClient != null) {
			return this.webView.viewClient.removeAuthenticationToken(host, realm);
		}
		return null;
	}

	/**
	 * Gets the authentication token.
	 * 
	 * In order it tries: 1- host + realm 2- host 3- realm 4- no host, no realm
	 * 
	 * @param host
	 * @param realm
	 * 
	 * @return the authentication token
	 */
	public AuthenticationToken getAuthenticationToken(String host, String realm) {
		if (this.webView != null && this.webView.viewClient != null) {
			return this.webView.viewClient.getAuthenticationToken(host, realm);
		}
		return null;
	}

	/**
	 * Clear all authentication tokens.
	 */
	public void clearAuthenticationTokens() {
		if (this.webView != null && this.webView.viewClient != null) {
			this.webView.viewClient.clearAuthenticationTokens();
		}
	}

	/**
	 * Called when the activity is first created.
	 * 
	 * @param savedInstanceState
	 */
	@Override
	public void onActivityCreate(Bundle savedInstanceState) {
		// preferences = new PreferenceSet();

		LOG.d(TAG, HybridView.class.getSimpleName() + ".onCreate()");

		this.setOrientation(LinearLayout.VERTICAL);
		this.setBackgroundColor(this.backgroundColor);
		this.setLayoutParams(new LinearLayout.LayoutParams(ViewGroup.LayoutParams.FILL_PARENT, ViewGroup.LayoutParams.FILL_PARENT));

		// Setup the hardware volume controls to handle volume control
		this.activity.setVolumeControlStream(AudioManager.STREAM_MUSIC);
	}

	/**
	 * Get the Android activity.
	 * 
	 * @return
	 */
	public Activity getActivity() {
		return this.activity;
	}
	
	/**
	 * 设置Activity内容
	 * @param activity
	 */
	public void setActivity(BaseActivity activity)
	{
		this.activity = activity;
	}

	/**
	 * Create and initialize web container with default web view objects.
	 */
	public void init() {
		HybridWebView webView = new HybridWebView(this);
		this.init(webView, new HybridWebViewClient(this, webView), new HybridChromeClient(this, webView));
	}

	/**
	 * Initialize web container with web view objects.
	 * 
	 * @param webView
	 * @param webViewClient
	 * @param webChromeClient
	 */
	public void init(HybridWebView webView, HybridWebViewClient webViewClient, HybridChromeClient webChromeClient) {
		LOG.d(TAG, HybridView.class.getSimpleName() + ".init()");

		// Set up web container
		this.webView = webView;
		this.webView.setId(100);
		this.webView.setWebViewClient(webViewClient);
		this.webView.setWebChromeClient(webChromeClient);
		webViewClient.setWebView(this.webView);
		webChromeClient.setWebView(this.webView);

		this.webView.setLayoutParams(new LinearLayout.LayoutParams(ViewGroup.LayoutParams.FILL_PARENT, ViewGroup.LayoutParams.FILL_PARENT));

		// Add web view but make it invisible while loading URL
		this.webView.setVisibility(View.VISIBLE);
		this.addView(this.webView);

		// Clear cancel flag
		this.cancelLoadUrl = false;
	}

	/**
	 * Load the url into the webview.
	 * 
	 * @param url
	 */
	public void loadUrl(String url) {

		// Init web view if not already done
		if (this.webView == null) {
			this.init();
		}

		// Set backgroundColor
		this.backgroundColor =Color.WHITE;
		this.webView.setBackgroundColor(this.backgroundColor);

		// If keepRunning
		this.keepRunning = IntentUtil.getBooleanProperty(this.activity.getIntent(), "keepRunning", true);

		// Then load the spinner
		this.loadSpinner();

		this.webView.loadUrl(url);
	}

	/*
	 * Load the spinner
	 */
	void loadSpinner() {

		// If loadingDialog property, then show the App loading dialog for first
		// page of app
		String loading = null;
		if ((this.webView == null) || !this.webView.canGoBack()) {
			loading = IntentUtil.getStringProperty(this.activity.getIntent(), "loadingDialog", null);
		} else {
			loading = IntentUtil.getStringProperty(this.activity.getIntent(), "loadingPageDialog", null);
		}
		if (loading != null) {

			String title = "";
			String message = "Loading Application...";

			if (loading.length() > 0) {
				int comma = loading.indexOf(',');
				if (comma > 0) {
					title = loading.substring(0, comma);
					message = loading.substring(comma + 1);
				} else {
					title = "";
					message = loading;
				}
			}
			this.spinnerStart(title, message);
		}
	}

	/**
	 * Load the url into the webview after waiting for period of time. This is
	 * used to display the splashscreen for certain amount of time.
	 * 
	 * @param url
	 * @param time
	 *            The number of ms to wait before loading webview
	 */
	public void loadUrl(final String url, int time) {

		// Init web view if not already done
		if (this.webView == null) {
			this.init();
		}

		this.splashscreenTime = time;
		this.webView.loadUrl(url, time);
	}

	/**
	 * Cancel loadUrl before it has been loaded.
	 */
	// TODO NO-OP
	@Deprecated
	public void cancelLoadUrl() {
		this.cancelLoadUrl = true;
	}

	/**
	 * Clear the resource cache.
	 */
	public void clearCache() {
		if (this.webView == null) {
			this.init();
		}
		this.webView.clearCache(true);
	}

	/**
	 * Clear web history in this web view.
	 */
	public void clearHistory() {
		this.webView.clearHistory();
	}
	
	/**
	 * 重新加载
	 */
	public void reload()
	{
		if (this.webView == null) {
			this.init();
		}
		this.webView.reload();
	}

	/**
	 * Go to previous page in history. (We manage our own history)
	 * 
	 * @return true if we went back, false if we are already at top
	 */
	public boolean backHistory() {
		if (this.webView != null) {
			return webView.backHistory();
		}
		return false;
	}


	@Override
	/**
	 * Called when the system is about to start resuming a previous activity.
	 */
	public void onActivityPause() {
		// Don't process pause if shutting down, since onDestroy() will be
		// called
		if (this.activityState == ACTIVITY_EXITING) {
			return;
		}

		if (this.webView == null) {
			return;
		}

		// Send pause event to JavaScript
		this.webView.loadUrl("javascript:try{cordova.fireDocumentEvent('pause');}catch(e){console.log('exception firing pause event from native');};");

		// Forward to plugins
		if (this.webView.pluginManager != null) {
			this.webView.pluginManager.onPause(this.keepRunning);
		}

		// If app doesn't want to run in background
		if (!this.keepRunning) {

			// Pause JavaScript timers (including setInterval)
			this.webView.pauseTimers();
		}
	}

	@Override
	/**
	 * Called when the activity receives a new intent
	 **/
	public void onNewIntent(Intent intent) {
		// Forward to plugins
		if ((this.webView != null) && (this.webView.pluginManager != null)) {
			this.webView.pluginManager.onNewIntent(intent);
		}
	}

	@Override
	/**
	 * Called when the activity will start interacting with the user.
	 */
	public void onActivityResume() {
		if (this.activityState == ACTIVITY_STARTING) {
			this.activityState = ACTIVITY_RUNNING;
			return;
		}

		if (this.webView == null) {
			return;
		}

		// Send resume event to JavaScript
		this.webView.loadUrl("javascript:try{cordova.fireDocumentEvent('resume');}catch(e){console.log('exception firing resume event from native');};");

		// Forward to plugins
		if (this.webView.pluginManager != null) {
			this.webView.pluginManager.onResume(this.keepRunning || this.activityResultKeepRunning);
		}

		// If app doesn't want to run in background
		if (!this.keepRunning || this.activityResultKeepRunning) {

			// Restore multitasking state
			if (this.activityResultKeepRunning) {
				this.keepRunning = this.activityResultKeepRunning;
				this.activityResultKeepRunning = false;
			}

			// Resume JavaScript timers (including setInterval)
			this.webView.resumeTimers();
		}
	}

	@Override
	/**
	 * The final call you receive before your activity is destroyed.
	 */
	public void onActivityDestroy() {
		LOG.d(TAG, "onDestroy()");

		if (this.webView != null) {

			// Send destroy event to JavaScript
			this.webView.loadUrl("javascript:try{cordova.require('cordova/channel').onDestroy.fire();}catch(e){console.log('exception firing destroy event from native');};");

			// Load blank page so that JavaScript onunload is called
//			this.webView.loadUrl("about:blank");

			// Forward to plugins
			if (this.webView.pluginManager != null) {
				this.webView.pluginManager.onDestroy();
			}
		} else {
			this.endActivity();
		}
	}

	/**
	 * Send a message to all plugins.
	 * 
	 * @param id
	 *            The message id
	 * @param data
	 *            The message data
	 */
	public void postMessage(String id, Object data) {
		if (this.webView != null) {
			this.webView.postMessage(id, data);
		}
	}

	/**
	 * @deprecated Add services to res/xml/plugins.xml instead.
	 * 
	 *             Add a class that implements a service.
	 * 
	 * @param serviceType
	 * @param className
	 */
	public void addService(String serviceType, String className) {
		if (this.webView != null && this.webView.pluginManager != null) {
			this.webView.pluginManager.addService(serviceType, className);
		}
	}

	/**
	 * Send JavaScript statement back to JavaScript. (This is a convenience
	 * method)
	 * 
	 * @param message
	 */
	public void sendJavascript(String statement) {
		if (this.webView != null && this.webView.callbackServer != null) {
			this.webView.callbackServer.sendJavascript(statement);
		}
	}

	/**
	 * Show the spinner. Must be called from the UI thread.
	 * 
	 * @param title
	 *            Title of the dialog
	 * @param message
	 *            The message of the dialog
	 */
	public void spinnerStart(final String title, final String message) {
		if (this.spinnerDialog != null) {
			this.spinnerDialog.dismiss();
			this.spinnerDialog = null;
		}
		final HybridView me = this;
		me.activity.runOnUiThread(new Runnable() {
			@Override
			public void run() {
				me.spinnerDialog = ProgressDialog.show(me.activity, title, message, true, true, new DialogInterface.OnCancelListener() {
					public void onCancel(DialogInterface dialog) {
						me.spinnerDialog = null;
					}
				});
			}
		});

	}

	/**
	 * Stop spinner.
	 */
	public void spinnerStop() {
		if (this.spinnerDialog != null) {
			this.spinnerDialog.dismiss();
			this.spinnerDialog = null;
		}
	}

	/**
	 * End this activity by calling finish for activity
	 */
	public void endActivity() {
		this.activityState = ACTIVITY_EXITING;
		this.activity.finish();
	}

	/**
	 * Launch an activity for which you would like a result when it finished.
	 * When this activity exits, your onActivityResult() method will be called.
	 * 
	 * @param command
	 *            The command object
	 * @param intent
	 *            The intent to start
	 * @param requestCode
	 *            The request code that is passed to callback to identify the
	 *            activity
	 */
	public void startActivityForResult(IPlugin command, Intent intent, int requestCode) {
		this.activityResultCallback = command;
		this.activityResultKeepRunning = this.keepRunning;

		// If multitasking turned on, then disable it for activities that return
		// results
		if (command != null) {
			this.keepRunning = false;
		}

		// Start activity
		this.activity.startActivityForResult(intent, requestCode);
	}

	@Override
	/**
	 * Called when an activity you launched exits, giving you the requestCode you started it with,
	 * the resultCode it returned, and any additional data from it.
	 *
	 * @param requestCode       The request code originally supplied to startActivityForResult(),
	 *                          allowing you to identify who this result came from.
	 * @param resultCode        The integer result code returned by the child activity through its setResult().
	 * @param data              An Intent, which can return result data to the caller (various data can be attached to Intent "extras").
	 */
	public void onActivityResult(int requestCode, int resultCode, Intent intent) {
		IPlugin callback = this.activityResultCallback;
		if (callback != null) {
			callback.onActivityResult(requestCode, resultCode, intent);
		}
	}

	public void setActivityResultCallback(IPlugin plugin) {
		this.activityResultCallback = plugin;
	}

	/**
	 * Report an error to the host application. These errors are unrecoverable
	 * (i.e. the main resource is unavailable). The errorCode parameter
	 * corresponds to one of the ERROR_* constants.
	 * 
	 * @param errorCode
	 *            The error code corresponding to an ERROR_* value.
	 * @param description
	 *            A String describing the error.
	 * @param failingUrl
	 *            The url that failed to load.
	 */
	public void onReceivedError(final int errorCode, final String description, final String failingUrl) {
		final HybridView me = this;

		// Stop "app loading" spinner if showing
		this.spinnerStop();

		// If errorUrl specified, then load it
		final String errorUrl = IntentUtil.getStringProperty(this.activity.getIntent(), "errorUrl", null);
		if ((errorUrl != null) && (errorUrl.startsWith("file://") || errorUrl.indexOf(me.baseUrl) == 0 || this.webView.isUrlWhiteListed(errorUrl)) && (!failingUrl.equals(errorUrl))) {

			// Load URL on UI thread
			this.activity.runOnUiThread(new Runnable() {
				public void run() {
					me.webView.showWebPage(errorUrl, false, true, null);
				}
			});
		}
		// If not, then display error dialog
		else {
			final boolean exit = !(errorCode == WebViewClient.ERROR_HOST_LOOKUP);
			this.activity.runOnUiThread(new Runnable() {
				public void run() {
					if (exit) {
						me.webView.setVisibility(View.GONE);
						me.displayError("Application Error", description + " (" + failingUrl + ")", "OK", exit);
					}
				}
			});
		}
	}

	/**
	 * Display an error dialog and optionally exit application.
	 * 
	 * @param title
	 * @param message
	 * @param button
	 * @param exit
	 */
	public void displayError(final String title, final String message, final String button, final boolean exit) {
		final HybridView me = this;
		this.activity.runOnUiThread(new Runnable() {
			public void run() {
				try {
					AlertDialog.Builder dlg = new AlertDialog.Builder(me.activity);
					dlg.setMessage(message);
					dlg.setTitle(title);
					dlg.setCancelable(false);
					dlg.setPositiveButton(button, new AlertDialog.OnClickListener() {
						public void onClick(DialogInterface dialog, int which) {
							dialog.dismiss();
							if (exit) {
								me.endActivity();
							}
						}
					});
					dlg.create();
					dlg.show();
				} catch (Exception e) {
					me.activity.finish();
				}
			}
		});
	}

	/**
	 * Determine if URL is in approved list of URLs to load.
	 * 
	 * @param url
	 * @return
	 */
	public boolean isUrlWhiteListed(String url) {
		// Check to see if we have matched url previously
		if (this.webView != null) {
			return this.webView.isUrlWhiteListed(url);
		}
		return false;
	}

	/*
	 * Hook in DroidGap for menu plugins
	 */
	@Override
	public ActionResult<Boolean> onCreateOptionsMenu(Menu menu) {
		this.postMessage("onCreateOptionsMenu", menu);
		return ActionResult.TRUE_AND_CONTINUE;
	}

	@Override
	public ActionResult<Boolean> onPrepareOptionsMenu(Menu menu) {
		this.postMessage("onPrepareOptionsMenu", menu);
		return ActionResult.TRUE_AND_CONTINUE;
	}

	@Override
	public ActionResult<Boolean> onOptionsItemSelected(MenuItem item) {
		this.postMessage("onOptionsItemSelected", item);
		return ActionResult.TRUE_AND_BREAK;
	}

	/**
	 * Load the specified URL in the Cordova webview or a new browser instance.
	 * 
	 * NOTE: If openExternal is false, only URLs listed in whitelist can be
	 * loaded.
	 * 
	 * @param url
	 *            The url to load.
	 * @param openExternal
	 *            Load url in browser instead of Cordova webview.
	 * @param clearHistory
	 *            Clear the history stack, so new page becomes top of history
	 * @param params
	 *            DroidGap parameters for new app
	 */
	public void showWebPage(String url, boolean openExternal, boolean clearHistory, HashMap<String, Object> params) {
		if (this.webView != null) {
			webView.showWebPage(url, openExternal, clearHistory, params);
		}
	}

	protected Dialog splashDialog;

	/**
	 * Removes the Dialog that displays the splash screen
	 */
	public void removeSplashScreen() {
		if (splashDialog != null) {
			splashDialog.dismiss();
			splashDialog = null;
		}
	}

	/**
	 * Shows the splash screen over the full Activity
	 */
	@SuppressWarnings("deprecation")
	protected void showSplashScreen(final int time) {
		final HybridView that = this;

		Runnable runnable = new Runnable() {
			public void run() {
				// Get reference to display
				Display display = that.activity.getWindowManager().getDefaultDisplay();

				// Create the layout for the dialog
				LinearLayout root = new LinearLayout(that.getActivity());
				root.setMinimumHeight(display.getHeight());
				root.setMinimumWidth(display.getWidth());
				root.setOrientation(LinearLayout.VERTICAL);
				root.setBackgroundColor(Color.WHITE);
				root.setLayoutParams(new LinearLayout.LayoutParams(ViewGroup.LayoutParams.FILL_PARENT, ViewGroup.LayoutParams.FILL_PARENT, 0.0F));
				root.setBackgroundResource(that.splashscreen);

				// Create and show the dialog
				splashDialog = new Dialog(that.activity, android.R.style.Theme_Translucent_NoTitleBar);
				// check to see if the splash screen should be full screen
				if ((that.activity.getWindow().getAttributes().flags & WindowManager.LayoutParams.FLAG_FULLSCREEN) == WindowManager.LayoutParams.FLAG_FULLSCREEN) {
					splashDialog.getWindow().setFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN, WindowManager.LayoutParams.FLAG_FULLSCREEN);
				}
				splashDialog.setContentView(root);
				splashDialog.setCancelable(false);
				splashDialog.show();

				// Set Runnable to remove splash screen just in case
				final Handler handler = new Handler();
				handler.postDelayed(new Runnable() {
					public void run() {
						removeSplashScreen();
					}
				}, time);
			}
		};
		this.activity.runOnUiThread(runnable);
	}

	/**
	 * Called when a message is sent to plugin.
	 * 
	 * @param id
	 *            The message id
	 * @param data
	 *            The message data
	 * @return Object or null
	 */
	public Object onMessage(String id, Object data) {
		LOG.d(TAG, "onMessage(" + id + "," + data + ")");
		if ("splashscreen".equals(id)) {
			if ("hide".equals(data.toString())) {
				this.removeSplashScreen();
			} else {
				this.splashscreen = IntentUtil.getIntegerProperty(this.activity.getIntent(), "splashscreen", 0);
				this.showSplashScreen(this.splashscreenTime);
			}
		} else if ("spinner".equals(id)) {
			if ("stop".equals(data.toString())) {
				this.spinnerStop();
				this.webView.setVisibility(View.VISIBLE);
			}
		} else if ("onReceivedError".equals(id)) {
			JSONObject d = (JSONObject) data;
			try {
				this.onReceivedError(d.getInt("errorCode"), d.getString("description"), d.getString("url"));
			} catch (JSONException e) {
				e.printStackTrace();
			}
		} else if ("exit".equals(id)) {
			this.endActivity();
		}
		return null;
	}

	@Override
	/**
	 * Start listening to new measurement events.  Fire events when the height
	 * gets smaller fire a show keyboard event and when height gets bigger fire
	 * a hide keyboard event.
	 *
	 * Note: We are using app.postMessage so that this is more compatible with the API
	 *
	 * @param widthMeasureSpec
	 * @param heightMeasureSpec
	 */
	protected void onMeasure(int widthMeasureSpec, int heightMeasureSpec) {
		super.onMeasure(widthMeasureSpec, heightMeasureSpec);

		LOG.v(TAG, "We are in our onMeasure method");

		// Get the current height of the visible part of the screen.
		// This height will not included the status bar.\
		int width, height;

		height = MeasureSpec.getSize(heightMeasureSpec);
		width = MeasureSpec.getSize(widthMeasureSpec);
		LOG.v(TAG, "Old Height = %d", oldHeight);
		LOG.v(TAG, "Height = %d", height);
		LOG.v(TAG, "Old Width = %d", oldWidth);
		LOG.v(TAG, "Width = %d", width);

		// If the oldHeight = 0 then this is the first measure event as the app
		// starts up.
		// If oldHeight == height then we got a measurement change that doesn't
		// affect us.
		if (oldHeight == 0 || oldHeight == height) {
			LOG.d(TAG, "Ignore this event");
		}
		// Account for orientation change and ignore this event/Fire orientation
		// change
		else if (screenHeight == width) {
			int tmp_var = screenHeight;
			screenHeight = screenWidth;
			screenWidth = tmp_var;
			LOG.v(TAG, "Orientation Change");
		}
		// If the height as gotten bigger then we will assume the soft keyboard
		// has
		// gone away.
		else if (height > oldHeight) {
			if (this.webView != null)
				this.webView.sendJavascript("cordova.fireDocumentEvent('hidekeyboard');");
		}
		// If the height as gotten smaller then we will assume the soft keyboard
		// has
		// been displayed.
		else if (height < oldHeight) {
			if (this.webView != null)
				this.webView.sendJavascript("cordova.fireDocumentEvent('showkeyboard');");
		}

		// Update the old height for the next event
		oldHeight = height;
		oldWidth = width;
	}

	@Override
	public ActionResult<Boolean> onActivityKeyDown(int keyCode, KeyEvent event) {
		return null;
	}

	@Override
	public void onActivityConfigurationChanged(Configuration newConfig) {
//		super.onConfigurationChanged(newConfig);
	}
	
	public void setBaseUrl(String baseUrl) {
		if(this.webView == null) {
			this.init();
		}
		this.webView.baseUrl = baseUrl;
	}
}
