package com.hundsun.hybrid.plugins;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import android.view.View;

import com.hundsun.hybrid.BaseActivity;
import com.hundsun.hybrid.HybridView;
import com.hundsun.hybrid.api.Hybrid;
import com.hundsun.hybrid.api.Plugin;
import com.hundsun.hybrid.api.PluginResult;
import com.hundsun.hybrid.utils.ResUtil;

public class InteractivePlugin extends Plugin {
	public static final String MESSAGE_ID = "interactive";
	private String callbackId;
	private BaseActivity activity;

	@Override
	public PluginResult execute(String action, JSONArray args, String callbackId) {
		PluginResult.Status status = PluginResult.Status.NO_RESULT;
		try {
			if ("init".equals(action)) {
				this.callbackId = callbackId;
				status = PluginResult.Status.OK;
				PluginResult result = new PluginResult(status, args);
				result.setKeepCallback(true);
				return result;
			} else if ("postMessage".equals(action)) {
				String viewId = args.getString(0);
				JSONObject data = args.getJSONObject(1);
				int id = ResUtil.getID(this.activity, viewId);
				HybridView view = (HybridView) this.activity.findViewById(id);
				view.postMessage(MESSAGE_ID, data);
			}
		} catch (JSONException e) {
			status = PluginResult.Status.JSON_EXCEPTION;
		}
		return new PluginResult(status);
	}
	

	@Override
	public boolean isSynch(String action) {
		if ("init".equals(action)) {
			return true;
		} else{
			return false;
		}
	}


	@Override
	public void setContext(Hybrid ctx) {
		super.setContext(ctx);
		if (!(ctx.getActivity() instanceof BaseActivity)) {
			throw new IllegalArgumentException("Activity is not an instance of BaseActivity");
		}
		this.activity = (BaseActivity) ctx.getActivity();
	}

	@Override
	public Object onMessage(String id, Object data) {
		if (MESSAGE_ID.equals(id)) {
			JSONObject json = (JSONObject) data;
			if (this.localProcess(json)) {
				return null;
			}
			PluginResult result = new PluginResult(PluginResult.Status.OK, json);
			result.setKeepCallback(true);
			this.success(result, this.callbackId);
		}
		return null;
	}

	private boolean localProcess(JSONObject json) {
		try {
			String callback = json.getString("callback");
			if ("loadUrl".equals(callback)) {
				String url = json.getString("args");
				this.webView.loadUrl(url);
				return true;
			}
			if ("setVisible".equals(callback)) {
				String args = json.getString("args");
				
				final HybridView view = (HybridView)this.hybrid;
				
				if("true".equals(args)){
					activity.runOnUiThread(new Runnable(){public void run(){view.setVisibility(View.VISIBLE);}});
				}else{
					activity.runOnUiThread(new Runnable(){public void run(){view.setVisibility(View.GONE);}});
				}
				return true;
			}
		} catch (JSONException e) {
			e.printStackTrace();
		}
		return false;
	}
}
