package com.hundsun.hybrid.plugins;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import android.util.Log;

import com.hundsun.hybrid.api.Plugin;
import com.hundsun.hybrid.api.PluginResult;

public class PageContext extends Plugin {
	public static final String TAG = PageContext.class.getSimpleName();

	/**
	 * Executes the request and returns PluginResult.
	 * 
	 * @param action
	 *            The action to execute.
	 * @param args
	 *            JSONArry of arguments for the plugin.
	 * @param callbackId
	 *            The callback id used when calling back into JavaScript.
	 * @return A PluginResult object with a status and message.
	 */
	public PluginResult execute(String action, JSONArray args, String callbackId) {
		Log.d(TAG, "execute action " + action + ", args " + args);
		PluginResult.Status status = PluginResult.Status.OK;
		String result = "";

		try {
			if ("getParameter".equals(action)) {
				if (args != null && args.length() > 0) {
					JSONObject obj = args.getJSONObject(0);
					String name = obj.getString("name");
					JSONObject json = this.webView.getParameter(name);
					return new PluginResult(PluginResult.Status.OK, json);
				}
			}
		} catch (JSONException e) {
			return new PluginResult(PluginResult.Status.JSON_EXCEPTION);
		}
		return new PluginResult(status, result);
	}

	/**
	 * Identifies if action to be executed returns a value and should be run
	 * synchronously.
	 * 
	 * @param action
	 *            The action to execute
	 * @return T=returns value
	 */
	public boolean isSynch(String action) {
		if ("getParameter".equals(action)) {
			return true;
		}
		return false;
	}

}
