package com.hundsun.hybrid.utils;

import android.content.Intent;
import android.os.Bundle;

public class IntentUtil {
	/**
	 * Get boolean property for activity.
	 * 
	 * @param name
	 * @param defaultValue
	 * @return
	 */
	public static boolean getBooleanProperty(Intent intent, String name, boolean defaultValue) {
		if (intent == null || name == null) {
			return defaultValue;
		}
		Bundle bundle = intent.getExtras();
		if (bundle == null) {
			return defaultValue;
		}
		Boolean p;
		try {
			p = (Boolean) bundle.get(name);
		} catch (ClassCastException e) {
			String s = bundle.get(name).toString();
			if ("true".equals(s)) {
				p = true;
			} else {
				p = false;
			}
		}
		if (p == null) {
			return defaultValue;
		}
		return p.booleanValue();
	}

	/**
	 * Get int property for activity.
	 * 
	 * @param name
	 * @param defaultValue
	 * @return
	 */
	public static int getIntegerProperty(Intent intent, String name, int defaultValue) {
		if (intent == null || name == null) {
			return defaultValue;
		}
		Bundle bundle = intent.getExtras();
		if (bundle == null) {
			return defaultValue;
		}
		Integer p;
		try {
			p = (Integer) bundle.get(name);
		} catch (ClassCastException e) {
			p = Integer.parseInt(bundle.get(name).toString());
		}
		if (p == null) {
			return defaultValue;
		}
		return p.intValue();
	}

	/**
	 * Get string property for activity.
	 * 
	 * @param name
	 * @param defaultValue
	 * @return
	 */
	public static String getStringProperty(Intent intent, String name, String defaultValue) {
		if (intent == null || name == null) {
			return defaultValue;
		}
		Bundle bundle = intent.getExtras();
		if (bundle == null) {
			return defaultValue;
		}
		String p = bundle.getString(name);
		if (p == null) {
			return defaultValue;
		}
		return p;
	}

	/**
	 * Get double property for activity.
	 * 
	 * @param name
	 * @param defaultValue
	 * @return
	 */
	public static double getDoubleProperty(Intent intent, String name, double defaultValue) {
		if (intent == null || name == null) {
			return defaultValue;
		}
		Bundle bundle = intent.getExtras();
		if (bundle == null) {
			return defaultValue;
		}
		Double p;
		try {
			p = (Double) bundle.get(name);
		} catch (ClassCastException e) {
			p = Double.parseDouble(bundle.get(name).toString());
		}
		if (p == null) {
			return defaultValue;
		}
		return p.doubleValue();
	}

	/**
	 * Set boolean property on activity.
	 * 
	 * @param name
	 * @param value
	 */
	public static void setBooleanProperty(Intent intent, String name, boolean value) {
		if (intent == null || name == null) {
			return;
		}
		intent.putExtra(name, value);
	}

	/**
	 * Set int property on activity.
	 * 
	 * @param name
	 * @param value
	 */
	public static void setIntegerProperty(Intent intent, String name, int value) {
		if (intent == null || name == null) {
			return;
		}
		intent.putExtra(name, value);
	}

	/**
	 * Set string property on activity.
	 * 
	 * @param name
	 * @param value
	 */
	public static void setStringProperty(Intent intent, String name, String value) {
		if (intent == null || name == null) {
			return;
		}
		intent.putExtra(name, value);
	}

	/**
	 * Set double property on activity.
	 * 
	 * @param name
	 * @param value
	 */
	public static void setDoubleProperty(Intent intent, String name, double value) {
		if (intent == null || name == null) {
			return;
		}
		intent.putExtra(name, value);
	}
}
