package com.hundsun.hybrid.utils;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import android.graphics.Color;

public class JsonUtils {
	/**
	 * 获取当前颜色
	 * 
	 * @param json
	 * @param key
	 * @return
	 */
	public static int getColor(JSONObject json, String key) {
		String colorStr = getStr(json, key);
		if (colorStr.length() == 9)
			return Color.parseColor(colorStr);
		else
			return Color.WHITE;
	}

	/** 获取JSON中的String字符串 **/
	public static String getStr(JSONObject json, String key) {
		return getStr(json, key, "");
	}

	public static String getStr(JSONObject json, String key, String defStr) {
		try {
			return json.getString(key);
		} catch (JSONException e) {
			return defStr;
		}
	}

	/** 获取JSON中的Long字符串 **/
	public static Long getLong(JSONObject json, String key) {
		return getLong(json, key, 0L);
	}

	public static Long getLong(JSONObject json, String key, long defLong) {
		try {
			return json.getLong(key);
		} catch (JSONException e) {
			return defLong;
		}
	}

	/** 获取JSON中的Int数字 **/
	public static int getInt(JSONObject json, String key) {
		return getInt(json, key, 0);
	}

	public static int getInt(JSONObject json, String key, int defInt) {
		try {
			return json.getInt(key);
		} catch (JSONException e) {
			return defInt;
		}
	}

	/** 获取JSON中的Boolean数字 **/
	public static boolean getBoolean(JSONObject json, String key) {
		return getBoolean(json, key, false);
	}

	public static boolean getBoolean(JSONObject json, String key, boolean defInt) {
		try {
			return json.getBoolean(key);
		} catch (JSONException e) {
			return defInt;
		}
	}

	public static float getFloat(JSONObject json, String key, float defaultValue) {
		try {
			return Float.parseFloat(json.getString(key));
		} catch (Exception e) {
			return defaultValue;
		}
	}

	public static double getDouble(JSONObject json, String key) {
		return getDouble(json, key, 0.00d);
	}

	public static double getDouble(JSONObject json, String key, double defInt) {
		try {
			return json.getDouble(key);
		} catch (JSONException e) {
			return defInt;
		}
	}

	public static JSONArray getArray(JSONObject json, String key) {
		try {
			JSONArray array = json.getJSONArray(key);
			if (array.length() == 0)
				return null;
			return array;
		} catch (JSONException e) {
			return null;
		}
	}

	public static JSONArray string2Array(String jsonStr) {
		JSONArray array = null;
		try {
			array = new JSONArray(jsonStr);
			return array;
		} catch (Exception e) {
			return null;
		}
	}
}
